package com.safeway.epe.controller;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;

import com.safeway.epe.domain.Offsets;
import com.safeway.epe.domain.TransactionPayload;
import com.safeway.epe.domain.TransactionRecorder;
import com.safeway.epe.repository.TransactionRepository;
import com.safeway.epe.service.ProducerService;

@SpringBootTest
@DisplayName("Component_Test_Case_For_Transaction_Controller")
public class TransactionControllerTest {

	Logger logger = LoggerFactory.getLogger(this.getClass());

	@Autowired
	private TransactionController transactionController;

	@MockBean
	private ProducerService producerService;

	@MockBean
	private TransactionRepository transactionRepository;

	private static String transactionId = "0ed78832-ea04-4062-b7dd-35c6c903c44c";
	private List<Offsets> listOfOffest;

	@Test
	public void testProducer_Success() {
		listOfOffest = new ArrayList<>();
		Offsets offsets = new Offsets();
		offsets.setPartition("0");
		offsets.setOffset("1");
		listOfOffest.add(offsets);

		UUID uuid = UUID.fromString(transactionId);
		Optional<TransactionRecorder> optionalOfferTransaction = Optional.of(createTransactionRecorder(uuid));
		Mockito.when(transactionRepository.findById(Mockito.any(UUID.class))).thenReturn(optionalOfferTransaction);
		Mockito.when(producerService.produce(Mockito.any(TransactionPayload.class), Mockito.anyString(),
				Mockito.any(TransactionRecorder.class), Mockito.anyBoolean())).thenReturn(listOfOffest);

		ResponseEntity<List<Offsets>> responseEntity = transactionController.produce(transactionId);
		Assertions.assertEquals(responseEntity.getStatusCode(), HttpStatus.OK);
	}

	@Test
	public void testProducer_Failure() {
		Optional<TransactionRecorder> optionalOfferTransaction = Optional.empty();
		Mockito.when(transactionRepository.findById(Mockito.any(UUID.class))).thenReturn(optionalOfferTransaction);
		ResponseEntity<List<Offsets>> responseEntity = transactionController.produce(transactionId);
		Assertions.assertEquals(responseEntity.getStatusCode(), HttpStatus.NOT_FOUND);
		Assertions.assertEquals(responseEntity.getBody(), Arrays.asList(new Offsets()));
	}

	private TransactionRecorder createTransactionRecorder(UUID uuid) {
		TransactionRecorder transactionRecorder = new TransactionRecorder();
		transactionRecorder.setUuid(uuid);
		transactionRecorder.setProcessed(true);
		transactionRecorder.setOffers("offers");
		transactionRecorder.setOfferTransactionResponse("TransactionResponse");
		return transactionRecorder;
	}

}
